/*
  Cloudfront Distribution: Content delivery network (CDN)
*/
resource "aws_cloudfront_distribution" "cdn_s3_distribution" {
  comment             = format("%s CDN - mcs2", var.cloudfront_name)
  aliases             = var.cloudfront_alias
  enabled             = var.enable_distribution
  is_ipv6_enabled     = var.enable_ipv6
  http_version        = var.http_version
  price_class         = var.price_class
  tags                = var.tags
  retain_on_delete    = var.retain_on_delete
  wait_for_deployment = var.wait_for_deployment
  web_acl_id          = var.web_acl_arn

  //  TODO: Configure this when the ACM certificate is ready
  //  viewer_certificate {
  //    acm_certificate_arn      = var.custom_viewer_certificate.acm_arn
  //    minimum_protocol_version = var.custom_viewer_certificate.minimum_protocol_version
  //    ssl_support_method       = var.custom_viewer_certificate.ssl_support_method
  //  }

  // TODO: Remove this when the ACM certificate is ready
  viewer_certificate {
    cloudfront_default_certificate = true
  }

  // The ISO 3166-1-alpha-2 codes for which you want CloudFront either to distribute or not distribute your content.
  restrictions {
    geo_restriction {
      restriction_type = var.geo_restriction_type
      locations        = var.geo_restriction_locations
    }
  }

  // Logging configuration, requires an s3 bucket
  logging_config {
    include_cookies = var.logging_includes_cookies
    bucket          = var.logging_bucket_domain_name
    prefix          = var.logging_prefix
  }

  /*
    Cloudfront Origin: An origin is the location where content is stored (S3),
    and from which Cloudfront gets content to serve when it's path pattern is requested.
  */
  // Default Origin
  origin {
    domain_name = var.default_bucket_domain_name
    origin_id   = local.default_s3_origin_id

    s3_origin_config {
      origin_access_identity = local.origin_id_path
    }
  }

  /*
    Cloudfront Cache Behaviors: An ordered list of cache behaviors resource for this distribution.
    List from top to bottom in order of precedence. The topmost cache behavior will have precedence 0.
    Includes a default cache behavior for when no path pattern for a request is found.
  */
  default_cache_behavior {
    target_origin_id       = local.default_s3_origin_id
    min_ttl                = var.default_cache_behavior.min_ttl
    default_ttl            = var.default_cache_behavior.default_ttl
    max_ttl                = var.default_cache_behavior.max_ttl
    viewer_protocol_policy = var.default_cache_behavior.viewer_protocol_policy
    allowed_methods        = var.default_cache_behavior.allowed_methods
    cached_methods         = var.default_cache_behavior.cached_methods
    compress               = var.default_cache_behavior.enable_compress

    # IDs that CloudFront can use to validate signed URLs/Cookies
    trusted_key_groups = var.default_cache_behavior.enable_signing ? [aws_cloudfront_key_group.trusted_keys.id] : []
    trusted_signers    = []

    forwarded_values {
      query_string = var.default_cache_behavior.fwd_query_string
      headers      = var.default_cache_behavior.fwd_headers

      cookies {
        forward = var.default_cache_behavior.fwd_cookies
      }
    }

    # Optional Lambda@Edge Configuration
    dynamic "lambda_function_association" {
      for_each = var.default_cache_behavior.enable_lambda ? [var.default_cache_lambda_arn] : []

      content {
        lambda_arn   = var.default_cache_lambda_arn
        event_type   = var.default_cache_behavior.event_type
        include_body = var.default_cache_behavior.include_body
      }
    }
  }

  /*
    Dynamic Cloudfront Origins & Cache Behaviors: An ordered list of cache behaviors resource for this distribution.
    Listed in order of precedence. The topmost cache behavior will have precedence 0.
  */
  // Dynamic Origins
  dynamic "origin" {
    for_each = var.cache_bucket_arns

    content {
      domain_name = var.cache_bucket_regional_domains[origin.key]
      origin_id   = var.cache_behaviors[origin.key].origin_id == "empty" ? regex(var.s3_regex_pattern, origin.value)[1] : var.cache_behaviors[origin.key].origin_id

      s3_origin_config {
        origin_access_identity = local.origin_id_path
      }
    }
  }

  dynamic "ordered_cache_behavior" {
    for_each = var.cache_behaviors

    content {
      target_origin_id       = var.cache_behaviors[ordered_cache_behavior.key].origin_id == "empty" ? regex(var.s3_regex_pattern, var.cache_bucket_arns[ordered_cache_behavior.key])[1] : var.cache_behaviors[ordered_cache_behavior.key].origin_id
      path_pattern           = var.cache_behaviors[ordered_cache_behavior.key].path_pattern
      min_ttl                = var.cache_behaviors[ordered_cache_behavior.key].min_ttl
      default_ttl            = var.cache_behaviors[ordered_cache_behavior.key].default_ttl
      max_ttl                = var.cache_behaviors[ordered_cache_behavior.key].max_ttl
      viewer_protocol_policy = var.cache_behaviors[ordered_cache_behavior.key].viewer_protocol_policy
      allowed_methods        = var.cache_behaviors[ordered_cache_behavior.key].allowed_methods
      cached_methods         = var.cache_behaviors[ordered_cache_behavior.key].cached_methods
      compress               = var.cache_behaviors[ordered_cache_behavior.key].enable_compress

      # IDs that CloudFront can use to validate signed URLs/Cookies
      trusted_key_groups = var.cache_behaviors[ordered_cache_behavior.key].enable_signing ? [aws_cloudfront_key_group.trusted_keys.id] : []
      trusted_signers    = []

      forwarded_values {
        query_string = var.cache_behaviors[ordered_cache_behavior.key].fwd_query_string
        headers      = var.cache_behaviors[ordered_cache_behavior.key].fwd_headers

        cookies {
          forward = var.cache_behaviors[ordered_cache_behavior.key].fwd_cookies
        }
      }

      # Optional Lambda@Edge Configuration
      dynamic "lambda_function_association" {
        for_each = var.cache_behaviors[ordered_cache_behavior.key].enable_lambda ? [var.cache_lambda_arns[ordered_cache_behavior.key]] : []

        content {
          lambda_arn   = var.cache_lambda_arns[ordered_cache_behavior.key]
          event_type   = var.cache_behaviors[ordered_cache_behavior.key].event_type
          include_body = var.cache_behaviors[ordered_cache_behavior.key].include_body
        }
      }
    }
  }
}
